use crate::{Context, Handler};
use axum::{extract::State, http::StatusCode, Json};
use serde::{Deserialize, Serialize};
use verax::{RevId, TxId};

#[derive(Deserialize)]
pub struct Lock {
    id: TxId,
    client_name: String,
}

#[derive(Serialize)]
pub struct Response {
    id: TxId,
    #[serde(rename = "_rev")]
    revision_id: RevId,
    secret: String,
}

#[async_trait::async_trait]
impl Handler for Lock {
    type Ok = Response;
    type Err = verax::Error;

    async fn handle(self, ledger: &Context) -> Result<Self::Ok, Self::Err> {
        let (new_tx, token) = ledger
            .ledger
            .lock_transaction(self.id, self.client_name)
            .await?;
        Ok(Response {
            id: new_tx.id,
            revision_id: new_tx.revision_id,
            secret: token.to_string(),
        })
    }
}

pub async fn handler(
    State(ctx): State<Context>,
    Json(item): Json<Lock>,
) -> Result<Json<Response>, (StatusCode, String)> {
    item.handle(&ctx)
        .await
        .map_err(|err| {
            (
                StatusCode::BAD_REQUEST,
                serde_json::to_string(&err).unwrap_or_default(),
            )
        })
        .map(Json)
}
